@@ -1480,15 +1480,15 @@ newt_shutdown(struct frontend *obj)
 
 struct frontend_module debconf_frontend_module =
 {
-initialize: newt_initialize,
-shutdown: newt_shutdown,
-lookup_directive: newt_lookup_directive,
-go: newt_go,
-can_go_back: newt_can_go_back,
-can_cancel_progress: newt_can_cancel_progress,
-can_align: newt_can_align,
-progress_start: newt_progress_start,
-progress_set:   newt_progress_set,
-progress_info:  newt_progress_info,
-progress_stop:  newt_progress_stop,
+.initialize = newt_initialize,
+.shutdown = newt_shutdown,
+.lookup_directive = newt_lookup_directive,
+.go = newt_go,
+.can_go_back = newt_can_go_back,
+.can_cancel_progress = newt_can_cancel_progress,
+.can_align = newt_can_align,
+.progress_start = newt_progress_start,
+.progress_set =   newt_progress_set,
+.progress_info =  newt_progress_info,
+.progress_stop =  newt_progress_stop,
 };
